#!/usr/bin/env python3
"""
Windows Network Configuration Manager (Network Management Tool)

Parameters:
    --list: List network adapters
    --info ADAPTER: Show adapter details
    --enable ADAPTER: Enable network adapter
    --disable ADAPTER: Disable network adapter
    --set-ip ADAPTER IP MASK: Set static IP
    --set-dhcp ADAPTER: Enable DHCP
    --format FORMAT: Output format (table/json)

Requires: win32api, wmi, tabulate packages and admin privileges
Warning: Incorrect network settings may cause connectivity loss
"""

import os
import sys
import logging
from typing import List, Dict, Any, Optional
import wmi
import win32com.client
from tabulate import tabulate
import json
import subprocess
import ipaddress
import argparse

class NetworkConfigManager:
    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self.wmi = wmi.WMI()
        self.shell = win32com.client.Dispatch("WScript.Shell")

    def get_network_adapters(self) -> List[Dict[str, Any]]:
        """Get all network adapters."""
        adapters = []
        try:
            for adapter in self.wmi.Win32_NetworkAdapter():
                if adapter.PhysicalAdapter:
                    adapters.append({
                        'name': adapter.Name,
                        'adapter_type': adapter.AdapterType,
                        'mac_address': adapter.MACAddress,
                        'status': adapter.Status,
                        'speed': adapter.Speed,
                        'enabled': adapter.NetEnabled
                    })
        except Exception as e:
            self.logger.error(f"Error getting network adapters: {str(e)}")
        return adapters

    def get_ip_config(self, adapter_name: Optional[str] = None) -> List[Dict[str, Any]]:
        """Get IP configuration for specified or all adapters."""
        configs = []
        try:
            for config in self.wmi.Win32_NetworkAdapterConfiguration(IPEnabled=True):
                if adapter_name and config.Description != adapter_name:
                    continue
                configs.append({
                    'adapter': config.Description,
                    'ip_addresses': config.IPAddress,
                    'subnet_masks': config.IPSubnet,
                    'default_gateway': config.DefaultIPGateway,
                    'dns_servers': config.DNSServerSearchOrder,
                    'dhcp_enabled': config.DHCPEnabled,
                    'dhcp_server': config.DHCPServer
                })
        except Exception as e:
            self.logger.error(f"Error getting IP configuration: {str(e)}")
        return configs

    def set_ip_config(self, adapter_name: str, ip_address: str,
                     subnet_mask: str, gateway: Optional[str] = None,
                     dns_servers: Optional[List[str]] = None) -> bool:
        """Set static IP configuration for an adapter."""
        try:
            # Validate IP addresses
            ipaddress.ip_address(ip_address)
            ipaddress.ip_address(subnet_mask)
            if gateway:
                ipaddress.ip_address(gateway)
            if dns_servers:
                for dns in dns_servers:
                    ipaddress.ip_address(dns)

            adapter = self.wmi.Win32_NetworkAdapterConfiguration(
                Description=adapter_name
            )[0]

            # Set IP address and subnet mask
            result = adapter.EnableStatic([ip_address], [subnet_mask])
            if result[0] != 0:
                self.logger.error("Failed to set IP configuration")
                return False

            # Set gateway if provided
            if gateway:
                result = adapter.SetGateways([gateway])
                if result[0] != 0:
                    self.logger.error("Failed to set gateway")
                    return False

            # Set DNS servers if provided
            if dns_servers:
                result = adapter.SetDNSServerSearchOrder(dns_servers)
                if result[0] != 0:
                    self.logger.error("Failed to set DNS servers")
                    return False

            return True
        except Exception as e:
            self.logger.error(f"Error setting IP configuration: {str(e)}")
            return False

    def enable_dhcp(self, adapter_name: str) -> bool:
        """Enable DHCP on specified adapter."""
        try:
            adapter = self.wmi.Win32_NetworkAdapterConfiguration(
                Description=adapter_name
            )[0]
            result = adapter.EnableDHCP()
            if result[0] != 0:
                self.logger.error("Failed to enable DHCP")
                return False
            return True
        except Exception as e:
            self.logger.error(f"Error enabling DHCP: {str(e)}")
            return False

    def get_firewall_rules(self) -> List[Dict[str, Any]]:
        """Get Windows Firewall rules."""
        rules = []
        try:
            firewall = win32com.client.Dispatch("HNetCfg.FwPolicy2")
            for rule in firewall.Rules:
                rules.append({
                    'name': rule.Name,
                    'description': rule.Description,
                    'protocol': rule.Protocol,
                    'local_ports': rule.LocalPorts,
                    'remote_ports': rule.RemotePorts,
                    'direction': 'Inbound' if rule.Direction == 1 else 'Outbound',
                    'action': 'Allow' if rule.Action == 1 else 'Block',
                    'enabled': rule.Enabled
                })
        except Exception as e:
            self.logger.error(f"Error getting firewall rules: {str(e)}")
        return rules

    def test_connectivity(self, target: str = '8.8.8.8') -> Dict[str, Any]:
        """Test network connectivity."""
        result = {
            'target': target,
            'success': False,
            'latency': None,
            'packet_loss': None
        }
        
        try:
            # Run ping command
            ping = subprocess.run(
                ['ping', '-n', '4', target],
                capture_output=True,
                text=True
            )
            
            output = ping.stdout.lower()
            
            # Parse results
            if 'bytes=32' in output:
                result['success'] = True
                
                # Extract latency
                for line in output.split('\n'):
                    if 'average' in line:
                        latency = line.split('=')[-1].strip('ms').strip()
                        result['latency'] = float(latency)
                        
                # Calculate packet loss
                sent = output.count('bytes=32')
                result['packet_loss'] = (4 - sent) / 4 * 100
                
        except Exception as e:
            self.logger.error(f"Error testing connectivity: {str(e)}")
            
        return result

    def display_results(self, data: Any, output_format: str = 'table'):
        """Display results in specified format."""
        if output_format == 'json':
            print(json.dumps(data, indent=4, default=str))
        else:
            self._display_table_format(data)

    def _display_table_format(self, data: Any):
        """Display data in table format."""
        if isinstance(data, list):
            if data and isinstance(data[0], dict):
                headers = data[0].keys()
                table_data = [item.values() for item in data]
                print(tabulate(table_data, headers=headers, tablefmt='grid'))
        elif isinstance(data, dict):
            table_data = [[k, str(v)] for k, v in data.items()]
            print(tabulate(table_data, headers=['Property', 'Value'], 
                         tablefmt='grid'))

def main():
    parser = argparse.ArgumentParser(
        description='Network Configuration Manager - Windows network utility')
    parser.add_argument('--list-adapters', action='store_true',
                       help='List all network adapters')
    parser.add_argument('--ip-config',
                       help='Get IP configuration for specified adapter')
    parser.add_argument('--set-static-ip', nargs=4,
                       metavar=('ADAPTER', 'IP', 'SUBNET', 'GATEWAY'),
                       help='Set static IP configuration')
    parser.add_argument('--set-dns', nargs='+',
                       help='Set DNS servers (use with --set-static-ip)')
    parser.add_argument('--enable-dhcp',
                       help='Enable DHCP on specified adapter')
    parser.add_argument('--firewall-rules', action='store_true',
                       help='List firewall rules')
    parser.add_argument('--test-connectivity',
                       help='Test network connectivity to target')
    parser.add_argument('--format', choices=['table', 'json'],
                       default='table',
                       help='Output format')
    
    args = parser.parse_args()
    
    # Setup logging
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s - %(levelname)s - %(message)s'
    )
    
    try:
        manager = NetworkConfigManager()
        
        if args.list_adapters:
            adapters = manager.get_network_adapters()
            manager.display_results(adapters, args.format)
            
        elif args.ip_config:
            configs = manager.get_ip_config(args.ip_config)
            manager.display_results(configs, args.format)
            
        elif args.set_static_ip:
            adapter, ip, subnet, gateway = args.set_static_ip
            dns_servers = args.set_dns if args.set_dns else None
            success = manager.set_ip_config(
                adapter, ip, subnet, gateway, dns_servers
            )
            print(f"{'Successfully' if success else 'Failed to'} "
                  f"set static IP configuration")
            
        elif args.enable_dhcp:
            success = manager.enable_dhcp(args.enable_dhcp)
            print(f"{'Successfully' if success else 'Failed to'} "
                  f"enable DHCP")
            
        elif args.firewall_rules:
            rules = manager.get_firewall_rules()
            manager.display_results(rules, args.format)
            
        elif args.test_connectivity:
            result = manager.test_connectivity(args.test_connectivity)
            manager.display_results(result, args.format)
            
        else:
            parser.print_help()
            
    except Exception as e:
        logging.error(f"An error occurred: {str(e)}")
        return 1
        
    return 0

if __name__ == "__main__":
    exit(main())
